@@ -239,9 +239,6 @@ show_popover_for_position (GcalMonthView *view,
 
   gint start_day, end_day;
 
-  icaltimetype *start_date;
-  icaltimetype *end_date = NULL;
-
   widget = GTK_WIDGET (view);
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
   ppriv = GCAL_SUBSCRIBER_VIEW (widget)->priv;
@@ -268,27 +265,23 @@ show_popover_for_position (GcalMonthView *view,
     }
   else
     {
-      start_date = gcal_dup_icaltime (priv->date);
-      start_date->day = start_day;
-      start_date->is_date = 1;
+      GDateTime *start_dt, *end_dt;
 
-      end_date = gcal_dup_icaltime (priv->date);
-      end_date->day = end_day;
-      end_date->is_date = 1;
+      start_dt = g_date_time_new_local (priv->date->year, priv->date->month, start_day, 0, 0, 0);
+      end_dt = g_date_time_new_local (priv->date->year, priv->date->month, end_day + 1, 0, 0, 0);
 
-      if (start_date->day > end_date->day)
+      /* Swap dates if start > end */
+      if (start_day > end_day)
         {
-          gint day = start_date->day;
-          start_date->day = end_date->day;
-          end_date->day = day;
+          GDateTime *aux = start_dt;
+          start_dt = end_dt;
+          end_dt = aux;
         }
 
-      end_date->day += 1;
+      g_signal_emit_by_name (GCAL_VIEW (widget), "create-event", start_dt, end_dt, x, y);
 
-      g_signal_emit_by_name (GCAL_VIEW (widget), "create-event", start_date, end_date, x, y);
-
-      g_free (start_date);
-      g_free (end_date);
+      g_date_time_unref (start_dt);
+      g_date_time_unref (end_dt);
     }
 
   gtk_widget_queue_draw (widget);
@@ -475,11 +468,6 @@ rebuild_popover_for_day (GcalMonthView *view,
   GdkRectangle rect;
 
   gchar *label_title;
-
-  icaltimetype date, second_date;
-  const icaltimetype *dt_start, *dt_end;
-  gint start_comparison, end_comparison;
-
   /* placement helpers */
   gdouble start_grid_y, cell_width, cell_height;
 
@@ -493,6 +481,8 @@ rebuild_popover_for_day (GcalMonthView *view,
   PangoFontDescription *ofont_desc;
   gint font_height, padding_bottom;
 
+  GTimeZone *tz;
+
   priv = gcal_month_view_get_instance_private (view);
   ppriv = GCAL_SUBSCRIBER_VIEW (view)->priv;
 
@@ -504,37 +494,45 @@ rebuild_popover_for_day (GcalMonthView *view,
   gtk_container_foreach (GTK_CONTAINER (priv->events_list_box), (GtkCallback) gtk_widget_destroy, NULL);
 
   l = g_hash_table_lookup (ppriv->overflow_cells, GINT_TO_POINTER (priv->pressed_overflow_indicator));
+  tz = g_time_zone_new_local ();
+
+  /* Setup the start & end dates of the events as the begin & end of day */
   if (l != NULL)
     {
-      date = *(priv->date);
-      date.day = day;
+      GDateTime *current_date;
+      GDateTime *dt_start;
+      GDateTime *dt_end;
+      current_date = icaltime_to_datetime (priv->date);
+      dt_start = g_date_time_new (tz,
+                                  g_date_time_get_year (current_date),
+                                  g_date_time_get_month (current_date),
+                                  day,
+                                  0, 0, 0);
 
-      second_date = date;
+      dt_end = g_date_time_new (tz,
+                                g_date_time_get_year (current_date),
+                                g_date_time_get_month (current_date),
+                                day + 1,
+                                0, 0, 0);
 
-      date.hour = 0;
-      date.minute = 0;
-      second_date.hour = 23;
-      second_date.minute =59;
+      for (; l != NULL; l = g_list_next (l))
+        {
+          GtkWidget *cloned_event;
+
+          cloned_event = gcal_event_widget_clone (l->data);
+          gcal_event_widget_set_date_start (GCAL_EVENT_WIDGET (cloned_event), dt_start);
+          gcal_event_widget_set_date_end (GCAL_EVENT_WIDGET (cloned_event), dt_end);
+
+          gtk_container_add (GTK_CONTAINER (priv->events_list_box), cloned_event);
+          _gcal_subscriber_view_setup_child (GCAL_SUBSCRIBER_VIEW (view), cloned_event);
+        }
+
+      g_clear_pointer (&current_date, g_date_time_unref);
+      g_clear_pointer (&dt_start, g_date_time_unref);
+      g_clear_pointer (&dt_end, g_date_time_unref);
     }
-  for (; l != NULL; l = g_list_next (l))
-    {
-      child_widget = gcal_event_widget_clone (GCAL_EVENT_WIDGET (l->data));
-      gtk_container_add (GTK_CONTAINER (priv->events_list_box), child_widget);
-      _gcal_subscriber_view_setup_child (GCAL_SUBSCRIBER_VIEW (view), child_widget);
 
-      dt_start = gcal_event_widget_peek_start_date (GCAL_EVENT_WIDGET (child_widget));
-      dt_end = gcal_event_widget_peek_end_date (GCAL_EVENT_WIDGET (child_widget));
-
-      date.is_date = dt_start->is_date;
-      start_comparison = icaltime_compare (*dt_start, date);
-      end_comparison = icaltime_compare (second_date, *dt_end);
-      if (start_comparison == -1 && end_comparison == -1)
-        gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted");
-      else if (start_comparison == -1)
-        gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-start");
-      else if (end_comparison == -1)
-        gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-end");
-    }
+  g_clear_pointer (&tz, g_time_zone_unref);
 
   /* placement calculation */
   start_grid_y = get_start_grid_y (GTK_WIDGET (view));
@@ -605,28 +603,33 @@ update_list_box_headers (GtkListBoxRow *row,
 {
   GcalMonthViewPrivate *priv;
   GtkWidget *row_child, *before_child = NULL;
-  const icaltimetype *row_date, *before_date = NULL;
+  GDateTime *row_date, *before_date = NULL;
+  GcalEvent *row_event;
 
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (user_data));
   row_child = gtk_bin_get_child (GTK_BIN (row));
-  row_date = gcal_event_widget_peek_start_date (GCAL_EVENT_WIDGET (row_child));
+  row_event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (row_child));
+  row_date = gcal_event_widget_get_date_start (GCAL_EVENT_WIDGET (row_child));
   if (before != NULL)
     {
       before_child = gtk_bin_get_child (GTK_BIN (before));
-      before_date = gcal_event_widget_peek_start_date (GCAL_EVENT_WIDGET (before_child));
+      before_date = gcal_event_widget_get_date_start (GCAL_EVENT_WIDGET (before_child));
     }
 
-  if (!gcal_event_widget_is_multiday (GCAL_EVENT_WIDGET (row_child)) &&
-      !gcal_event_widget_get_all_day (GCAL_EVENT_WIDGET (row_child)) &&
-      (before_date == NULL || before_date->hour != row_date->hour))
+  if (!gcal_event_is_multiday (row_event) &&
+      !gcal_event_get_all_day (row_event) &&
+      (before_date == NULL || g_date_time_get_hour (before_date) != g_date_time_get_hour (row_date)))
     {
       gchar *time;
       GtkWidget *label, *vbox;
+      gint hour;
+
+      hour = g_date_time_get_hour (row_date);
 
       if (priv->use_24h_format)
-        time = g_strdup_printf ("%.2d:00", row_date->hour);
+        time = g_strdup_printf ("%.2d:00", hour);
       else
-        time = g_strdup_printf ("%.2d:00 %s", row_date->hour % 12, row_date->hour < 12 ? "AM" : "PM");
+        time = g_strdup_printf ("%.2d:00 %s", hour % 12, hour < 12 ? "AM" : "PM");
 
       label = gtk_label_new (time);
       gtk_style_context_add_class (gtk_widget_get_style_context (label), GTK_STYLE_CLASS_DIM_LABEL);
@@ -738,19 +741,18 @@ add_new_event_button_cb (GtkWidget *button,
 {
   GcalMonthViewPrivate *priv;
   gint day;
-  icaltimetype *start_date;
+  GDateTime *start_date;
 
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (user_data));
 
   gtk_widget_hide (priv->overflow_popover);
 
   day = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (priv->overflow_popover), "selected-day"));
-  start_date = gcal_dup_icaltime (priv->date);
-  start_date->day = day;
-  start_date->is_date = 1;
+  start_date = g_date_time_new_local (priv->date->year, priv->date->month, day, 0, 0, 0);
 
   g_signal_emit_by_name (GCAL_VIEW (user_data), "create-event-detailed", start_date, NULL);
-  g_free (start_date);
+
+  g_date_time_unref (start_date);
 }
 
 static void
@@ -847,8 +849,10 @@ gcal_month_view_set_property (GObject       *object,
                               const GValue  *value,
                               GParamSpec    *pspec)
 {
+  GcalSubscriberViewPrivate *ppriv;
   GcalMonthViewPrivate *priv;
 
+  ppriv = GCAL_SUBSCRIBER_VIEW (object)->priv;
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (object));
 
   switch (property_id)
@@ -863,7 +867,9 @@ gcal_month_view_set_property (GObject       *object,
         priv->keyboard_cell = priv->days_delay;
         priv->start_mark_cell = -1;
         priv->end_mark_cell = -1;
-        gtk_widget_queue_draw (GTK_WIDGET (object));
+
+        ppriv->children_changed = TRUE;
+        gtk_widget_queue_resize (GTK_WIDGET (object));
         break;
       }
     default:
@@ -1017,6 +1023,7 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
   context = gtk_widget_get_style_context (widget);
   ppriv = GCAL_SUBSCRIBER_VIEW (widget)->priv;
 
+  /* No need to relayout stuff if nothing changed */
   if (!ppriv->children_changed &&
       allocation->height == gtk_widget_get_allocated_height (widget) &&
       allocation->width == gtk_widget_get_allocated_width (widget))
@@ -1035,12 +1042,17 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
   /* clean overflow information */
   g_hash_table_remove_all (ppriv->overflow_cells);
 
+  /* Allocate the widget */
   gtk_widget_set_allocation (widget, allocation);
+
   if (gtk_widget_get_realized (widget))
     gdk_window_move_resize (priv->event_window, allocation->x, allocation->y, allocation->width, allocation->height);
 
-  gtk_style_context_get (context, gtk_style_context_get_state (context),
-                         "font", &font_desc, "padding-bottom", &padding_bottom, NULL);
+  gtk_style_context_get (context,
+                         gtk_style_context_get_state (context),
+                         "font", &font_desc,
+                         "padding-bottom", &padding_bottom,
+                         NULL);
 
   layout = gtk_widget_create_pango_layout (widget, _("Other events"));
   pango_layout_set_font_description (layout, font_desc);
@@ -1060,72 +1072,87 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
   first_row_gap = (6 - shown_rows) * 0.5;
   sw = 1 - 2 * priv->k;
 
-  /* allocate multidays events */
+  /*
+   * Allocate multidays events before single day events, as they have a
+   * higher priority. Multiday events have a master event, and clones
+   * to represent the multiple sections of the event.
+   */
   for (l = ppriv->multi_cell_children; l != NULL; l = g_list_next (l))
     {
       gint first_cell, last_cell, first_row, last_row, start, end;
-      gboolean visible, start_before = TRUE, end_after = TRUE;
+      gboolean visible;
 
-      const icaltimetype *date;
+      GDateTime *date;
+      GcalEvent *event;
       GArray *cells, *lengths;
 
       child_widget = (GtkWidget*) l->data;
-      uuid = gcal_event_widget_peek_uuid (GCAL_EVENT_WIDGET (child_widget));
+      event = gcal_event_widget_get_event (l->data);
+      uuid = gcal_event_get_uid (event);
+
       if (!gtk_widget_is_visible (child_widget) && !g_hash_table_contains (ppriv->hidden_as_overflow, uuid))
         continue;
 
       gtk_widget_show (child_widget);
       gtk_widget_get_preferred_height (child_widget, NULL, &natural_height);
 
+      /*
+       * If the month of the event's start date is equal to the month
+       * we're visualizing, the first cell is the event's day of the
+       * month. Otherwise, the first cell is the 1st day of the month.
+       */
       j = 1;
-      date = gcal_event_widget_peek_start_date (GCAL_EVENT_WIDGET (child_widget));
-      if (date->month == priv->date->month)
-        {
-          j = date->day;
-          start_before = FALSE;
-        }
+      date = gcal_event_get_date_start (event);
+
+      if (g_date_time_get_month (date) == priv->date->month)
+        j = g_date_time_get_day_of_month (date);
+
       j += priv->days_delay;
+
+      /*
+       * Calculate the first cell position according to the locale
+       * and the start date.
+       */
       first_cell = 7 * ((j - 1) / 7)+ 6 * priv->k + sw * ((j - 1) % 7);
 
+      /*
+       * The logic for the end date is the same, except that we have to check
+       * if the event is all day or not.
+       */
       j = icaltime_days_in_month (priv->date->month, priv->date->year);
-      date = gcal_event_widget_peek_end_date (GCAL_EVENT_WIDGET (child_widget));
-      if (gcal_event_widget_get_all_day (GCAL_EVENT_WIDGET (child_widget)))
+      date = gcal_event_get_date_end (event);
+
+      if (g_date_time_get_month (date) == priv->date->month)
         {
-          if (date->month == priv->date->month)
-            {
-              j = date->day - 1;
-              end_after = FALSE;
-            }
-        }
-      else
-        {
-          if (date->month == priv->date->month)
-            {
-              j = date->day;
-              end_after = FALSE;
-            }
+          j = g_date_time_get_day_of_month (date);
+
+          /* If the event is all day, we have to subtract 1 to find the the real date */
+          if (gcal_event_get_all_day (event))
+            j--;
         }
       j += priv->days_delay;
+
       last_cell = 7 * ((j - 1) / 7)+ 6 * priv->k + sw * ((j - 1) % 7);
 
+      /*
+       * Now that we found the start & end dates, we have to find out
+       * whether the event is visible or not.
+       */
       first_row = first_cell / 7;
       last_row = last_cell / 7;
       visible = TRUE;
       cells = g_array_sized_new (TRUE, TRUE, sizeof (gint), 16);
       lengths = g_array_sized_new (TRUE, TRUE, sizeof (gint), 16);
+
       for (i = first_row; i <= last_row && visible; i++)
         {
           start = i * 7 + priv->k * 6;
           end = i * 7 + (1 - priv->k) * 6;
 
           if (i == first_row)
-            {
-              start = first_cell;
-            }
+            start = first_cell;
           if (i == last_row)
-            {
-              end = last_cell;
-            }
+            end = last_cell;
 
           visible = get_widget_parts (start, end, natural_height, vertical_cell_space, size_left, cells, lengths);
         }
@@ -1134,9 +1161,16 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
         {
           for (i = 0; i < cells->len; i++)
             {
+              GDateTime *dt_start, *dt_end;
               gint cell_idx = g_array_index (cells, gint, i);
+              gint length = g_array_index (lengths, gint, i);
               gint row = cell_idx / 7;
               gint column = cell_idx % 7;
+              gint day = cell_idx - priv->days_delay + 1;
+
+              /* Day number is calculated differently on RTL languages */
+              if (priv->k)
+                day = 7 * row + MIRROR (day % 7, 0, 7) - length;
 
               if (i != 0)
                 {
@@ -1145,29 +1179,31 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
                   _gcal_subscriber_view_setup_child (GCAL_SUBSCRIBER_VIEW (widget), child_widget);
                   gtk_widget_show (child_widget);
 
-                  if (i != cells->len - 1 || end_after)
-                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted");
-                  else
-                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-start");
-
                   aux = g_hash_table_lookup (ppriv->children, uuid);
                   aux = g_list_append (aux, child_widget);
                 }
-              else if (i == 0 && cells->len == 1)
-                {
-                  if (start_before)
-                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-start");
-                  if (end_after)
-                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-end");
-                }
-              else if (i != cells->len - 1)
-                {
-                  if (start_before)
-                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted");
-                  else
-                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-end");
-                }
 
+              /* XXX: silence Gtk+ complaining about preferred size */
+              gtk_widget_get_preferred_height (child_widget, NULL, &natural_height);
+
+              /*
+               * Setup the widget's start date as the first day of the row,
+               * and the widget's end date as the last day of the row. We don't
+               * have to worry about the dates, since GcalEventWidget performs
+               * some checks and only applies the dates when it's valid.
+               */
+              dt_start = g_date_time_new (gcal_event_get_timezone (event),
+                                          priv->date->year,
+                                          priv->date->month,
+                                          day,
+                                          0, 0, 0);
+
+              dt_end = g_date_time_add_days (dt_start, length);
+
+              gcal_event_widget_set_date_start (GCAL_EVENT_WIDGET (child_widget), dt_start);
+              gcal_event_widget_set_date_end (GCAL_EVENT_WIDGET (child_widget), dt_end);
+
+              /* Position and allocate the child widget */
               gtk_style_context_get_margin (gtk_widget_get_style_context (child_widget),
                                             gtk_style_context_get_state (context),
                                             &margin);
@@ -1177,15 +1213,18 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
 
               child_allocation.x = pos_x;
               child_allocation.y = pos_y + vertical_cell_space - size_left[cell_idx];
-              child_allocation.width = cell_width * g_array_index (lengths, gint, i) - (margin.left + margin.right);
+              child_allocation.width = cell_width * length - (margin.left + margin.right);
               child_allocation.height = natural_height;
 
               gtk_widget_size_allocate (child_widget, &child_allocation);
               g_hash_table_remove (ppriv->hidden_as_overflow, uuid);
 
               /* update size_left */
-              for (j = 0; j < g_array_index (lengths, gint, i); j++)
+              for (j = 0; j < length; j++)
                 size_left[cell_idx + j] -= natural_height + margin.top + margin.bottom;
+
+              g_clear_pointer (&dt_start, g_date_time_unref);
+              g_clear_pointer (&dt_end, g_date_time_unref);
             }
         }
       else
@@ -1209,7 +1248,13 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
       g_array_free (lengths, TRUE);
     }
 
+  /*
+   * Allocate single day events after multiday ones. For single
+   * day children, there's no need to calculate the start & end
+   * dates of the widgets.
+   */
   g_hash_table_iter_init (&iter, ppriv->single_cell_children);
+
   while (g_hash_table_iter_next (&iter, &key, &value))
     {
       j = GPOINTER_TO_INT (key);
@@ -1221,7 +1266,8 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
         {
           child_widget = (GtkWidget*) aux->data;
 
-          uuid = gcal_event_widget_peek_uuid (GCAL_EVENT_WIDGET (child_widget));
+          uuid = gcal_event_get_uid (gcal_event_widget_get_event (GCAL_EVENT_WIDGET (child_widget)));
+
           if (!gtk_widget_is_visible (child_widget) && !g_hash_table_contains (ppriv->hidden_as_overflow, uuid))
             continue;
 
@@ -1797,15 +1843,18 @@ static gboolean
 gcal_month_view_is_child_multiday (GcalSubscriberView *subscriber,
                                      GcalEventWidget    *child)
 {
-  return gcal_event_widget_is_multiday (child);
+  return gcal_event_is_multiday (gcal_event_widget_get_event (child));
 }
 
 static guint
 gcal_month_view_get_child_cell (GcalSubscriberView *subscriber,
                                 GcalEventWidget    *child)
 {
-  const icaltimetype *dt_start = gcal_event_widget_peek_start_date (child);
-  return dt_start->day;
+  GDateTime *dt;
+
+  dt = gcal_event_widget_get_date_start (child);
+
+  return g_date_time_get_day_of_month (dt);
 }
 
 static void
@@ -1872,7 +1921,7 @@ gcal_month_view_get_final_date (GcalView *view)
   new_date->is_date = 0;
   new_date->hour = 23;
   new_date->minute = 59;
-  new_date->second = 0;
+  new_date->second = 59;
 
   return new_date;
 }
