@@ -63,6 +63,80 @@ month_item[12] =
 
 G_DEFINE_BOXED_TYPE (icaltimetype, icaltime, gcal_dup_icaltime, g_free)
 
+gint
+datetime_compare_date (GDateTime *dt1,
+                       GDateTime *dt2)
+{
+  if (!dt1 && !dt2)
+    return 0;
+  else if (!dt1)
+    return -1;
+  else if (!dt2)
+    return 1;
+
+  if (g_date_time_get_year (dt1) != g_date_time_get_year (dt2))
+    return g_date_time_get_year (dt1) - g_date_time_get_year (dt2);
+
+  if (g_date_time_get_month (dt1) != g_date_time_get_month (dt2))
+    return g_date_time_get_month (dt1) - g_date_time_get_month (dt2);
+
+  if (g_date_time_get_day_of_month (dt1) != g_date_time_get_day_of_month (dt2))
+    return g_date_time_get_day_of_month (dt1) - g_date_time_get_day_of_month (dt2);
+
+  return 0;
+}
+
+icaltimetype*
+datetime_to_icaltime (GDateTime *dt)
+{
+  icaltimetype *idt;
+
+  if (!dt)
+    return NULL;
+
+  idt = g_new0 (icaltimetype, 1);
+
+  idt->year = g_date_time_get_year (dt);
+  idt->month = g_date_time_get_month (dt);
+  idt->day = g_date_time_get_day_of_month (dt);
+  idt->hour = g_date_time_get_hour (dt);
+  idt->minute = g_date_time_get_minute (dt);
+  idt->second = g_date_time_get_seconds (dt);
+  idt->is_date = (idt->hour == 0 &&
+                  idt->minute == 0 &&
+                  idt->second == 0);
+
+  return idt;
+}
+
+GDateTime*
+icaltime_to_datetime (const icaltimetype  *date)
+{
+  GDateTime *dt;
+  GTimeZone *tz;
+
+  tz = date->zone ? g_time_zone_new (icaltime_get_tzid (*date)) : g_time_zone_new_utc ();
+  dt = g_date_time_new (tz,
+                        date->year,
+                        date->month,
+                        date->day,
+                        date->is_date ? 0 : date->hour,
+                        date->is_date ? 0 : date->minute,
+                        date->is_date ? 0 : date->second);
+
+  g_clear_pointer (&tz, g_time_zone_unref);
+
+  return dt;
+}
+
+gboolean
+datetime_is_date (GDateTime *dt)
+{
+  return g_date_time_get_hour (dt) == 0 &&
+         g_date_time_get_minute (dt) == 0 &&
+         g_date_time_get_seconds (dt) == 0;
+}
+
 icaltimetype*
 gcal_dup_icaltime (const icaltimetype *date)
 {
@@ -178,22 +252,18 @@ gcal_compare_event_widget_by_date (gconstpointer a,
   /* negative value if a < b; zero if a = b; positive value if a > b. */
   GcalViewChild *a_child;
   GcalViewChild *b_child;
-  icaltimetype *a_date;
-  icaltimetype *b_date;
+  GDateTime *a_date;
+  GDateTime *b_date;
 
   gint comparison;
 
   a_child = (GcalViewChild*) a;
   b_child = (GcalViewChild*) b;
 
-  a_date =
-    gcal_event_widget_get_date (GCAL_EVENT_WIDGET (a_child->widget));
-  b_date =
-    gcal_event_widget_get_date (GCAL_EVENT_WIDGET (b_child->widget));
+  a_date = gcal_event_get_date_start (gcal_event_widget_get_event (GCAL_EVENT_WIDGET (a_child->widget)));
+  b_date = gcal_event_get_date_start (gcal_event_widget_get_event (GCAL_EVENT_WIDGET (b_child->widget)));
 
-  comparison = icaltime_compare (*a_date, *b_date);
-  g_free (a_date);
-  g_free (b_date);
+  comparison = g_date_time_compare (a_date, b_date);
 
   return comparison;
 }
@@ -346,35 +416,49 @@ get_first_weekday (void)
  * Returns: (Transfer full): an {@link ECalComponent} object
  **/
 ECalComponent*
-build_component_from_details (const gchar        *summary,
-                              const icaltimetype *initial_date,
-                              const icaltimetype *final_date)
+build_component_from_details (const gchar *summary,
+                              GDateTime   *initial_date,
+                              GDateTime   *final_date)
 {
   ECalComponent *event;
-
   ECalComponentDateTime dt;
   ECalComponentText summ;
+  GDateTime *utc_start, *utc_end;
+  gboolean all_day;
 
   event = e_cal_component_new ();
   e_cal_component_set_new_vtype (event, E_CAL_COMPONENT_EVENT);
 
-  dt.value = (icaltimetype*) initial_date;
-  dt.tzid = NULL;
+  /*
+   * Check if the event is all day. Notice that it can be all day even
+   * without the final date.
+   */
+  all_day = datetime_is_date (initial_date) && (final_date ? datetime_is_date (final_date) : TRUE);
+
+  /* Start date */
+  utc_start = g_date_time_to_utc (initial_date);
+
+  dt.value = datetime_to_icaltime (utc_start);
+  dt.value->is_date = all_day;
+  dt.tzid = format_utc_offset (g_date_time_get_utc_offset (initial_date));
   e_cal_component_set_dtstart (event, &dt);
 
-  if (final_date != NULL)
-    {
-      dt.value = (icaltimetype*) final_date;
-      e_cal_component_set_dtend (event, &dt);
-    }
-  else
-    {
-      icaltimetype *dt_end = gcal_dup_icaltime (initial_date);
-      icaltime_adjust (dt_end, 1, 0, 0, 0);
-      dt.value = dt_end;
-      e_cal_component_set_dtend (event, &dt);
-      g_free (dt_end);
-    }
+  e_cal_component_free_datetime (&dt);
+  g_date_time_unref (utc_start);
+
+  /* End date */
+  if (!final_date)
+    final_date = g_date_time_add_days (initial_date, 1);
+
+  utc_end = g_date_time_to_utc (final_date);
+
+  dt.value = datetime_to_icaltime (utc_end);
+  dt.value->is_date = all_day;
+  dt.tzid = format_utc_offset (g_date_time_get_utc_offset (final_date));
+  e_cal_component_set_dtend (event, &dt);
+
+  e_cal_component_free_datetime (&dt);
+  g_date_time_unref (utc_end);
 
   summ.altrep = NULL;
   summ.value = summary;
@@ -707,3 +791,24 @@ get_source_parent_name_color (GcalManager  *manager,
       *color = gdk_rgba_to_string (&c);
     }
 }
+
+gchar*
+format_utc_offset (gint offset)
+{
+  const char *sign = "+";
+  gint hours, minutes, seconds;
+
+  if (offset < 0) {
+      offset = -offset;
+      sign = "-";
+  }
+
+  hours = offset / 3600;
+  minutes = (offset % 3600) / 60;
+  seconds = offset % 60;
+
+  if (seconds > 0)
+    return g_strdup_printf ("%s%02i%02i%02i", sign, hours, minutes, seconds);
+  else
+    return g_strdup_printf ("%s%02i%02i", sign, hours, minutes);
+}
