@@ -52,6 +52,15 @@ const gchar*  (*GcalTranslateFunc)                              (GtkWidget
 
 GType           icaltime_get_type                               (void)            G_GNUC_CONST;
 
+gint            datetime_compare_date                           (GDateTime             *dt1,
+                                                                 GDateTime             *dt2);
+
+icaltimetype*   datetime_to_icaltime                            (GDateTime             *dt);
+
+gboolean        datetime_is_date                                (GDateTime             *dt);
+
+GDateTime*      icaltime_to_datetime                            (const icaltimetype    *date);
+
 icaltimetype*   gcal_dup_icaltime                               (const icaltimetype    *date);
 
 gchar*          gcal_get_weekday                                (gint                   i);
@@ -79,8 +88,8 @@ gchar*          get_uuid_from_component                         (ESource
 gint            get_first_weekday                               (void);
 
 ECalComponent*  build_component_from_details                    (const gchar           *summary,
-                                                                 const icaltimetype    *initial_date,
-                                                                 const icaltimetype    *final_date);
+                                                                 GDateTime             *initial_date,
+                                                                 GDateTime             *final_date);
 
 gint            icaltime_compare_date                           (const icaltimetype    *date1,
                                                                  const icaltimetype    *date2);
@@ -112,4 +121,6 @@ void            get_source_parent_name_color                    (GcalManager
                                                                  gchar                **name,
                                                                  gchar                **color);
 
+gchar*          format_utc_offset                               (gint                   offset);
+
 #endif // __GCAL_UTILS_H__
